@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import logging
 import os
 import socket
@@ -5,7 +7,6 @@ import sys
 import time
 from argparse import ArgumentParser
 from io import StringIO
-from typing import Optional
 
 from blessed import Terminal
 
@@ -14,7 +15,7 @@ from .config import Configuration, ConfigurationError
 from .pg import OperationalError
 
 
-def configure_logger(debug_file: Optional[str] = None) -> StringIO:
+def configure_logger(debug_file: str | None = None) -> StringIO:
     logger = logging.getLogger("pgactivity")
     logger.setLevel(logging.DEBUG)
 
@@ -59,6 +60,19 @@ def get_parser() -> ArgumentParser:
         add_help=False,
     )
 
+    group = parser.add_argument_group(
+        "Configuration",
+    )
+    group.add_argument(
+        "-P",
+        "--profile",
+        help=(
+            "Configuration profile matching a PROFILE.conf file in "
+            "${XDG_CONFIG_HOME:~/.config}/pg_activity/ or /etc/pg_activity/, "
+            "or a built-in profile."
+        ),
+    )
+
     group = parser.add_argument_group(
         "Options",
     )
@@ -314,6 +328,32 @@ def get_parser() -> ArgumentParser:
         default=False,
     )
 
+    group = parser.add_argument_group("Header display options")
+    # --no-inst-info
+    group.add_argument(
+        "--no-inst-info",
+        dest="header_show_instance",
+        action="store_false",
+        help="Hide instance information.",
+        default=None,
+    )
+    # --no-sys-info
+    group.add_argument(
+        "--no-sys-info",
+        dest="header_show_system",
+        action="store_false",
+        help="Hide system information.",
+        default=None,
+    )
+    # --no-proc-info
+    group.add_argument(
+        "--no-proc-info",
+        dest="header_show_workers",
+        action="store_false",
+        help="Hide workers process information.",
+        default=None,
+    )
+
     group = parser.add_argument_group("Other display options")
     # --hide-queries-in-logs
     group.add_argument(
@@ -323,30 +363,6 @@ def get_parser() -> ArgumentParser:
         help="Disable log_min_duration_statements and log_min_duration_sample for pg_activity.",
         default=False,
     )
-    # --no-inst-info
-    group.add_argument(
-        "--no-inst-info",
-        dest="show_instance_info_in_header",
-        action="store_false",
-        help="Display instance information in header.",
-        default=True,
-    )
-    # --no-sys-info
-    group.add_argument(
-        "--no-sys-info",
-        dest="show_system_info_in_header",
-        action="store_false",
-        help="Display system information in header.",
-        default=True,
-    )
-    # --no-proc-info
-    group.add_argument(
-        "--no-proc-info",
-        dest="show_worker_info_in_header",
-        action="store_false",
-        help="Display workers process information in header.",
-        default=True,
-    )
     # --refresh
     group.add_argument(
         "--refresh",
@@ -387,16 +403,14 @@ def main() -> None:
         args.notempfile = True
 
     try:
-        cfg = Configuration.lookup()
-    except ConfigurationError as e:
+        cfg = Configuration.lookup(args.profile)
+    except (ConfigurationError, FileNotFoundError) as e:
         parser.error(str(e))
 
-    dataobj = data.pg_connect(
-        args,
-        args.connection_string,
-        min_duration=args.minduration,
-        filters=filters,
-    )
+    try:
+        dataobj = data.pg_connect(args, min_duration=args.minduration, filters=filters)
+    except OperationalError as e:
+        parser.exit(status=1, message=f"could not connect to PostgreSQL: {e}")
     hostname = socket.gethostname()
     conninfo = dataobj.pg_conn.info
     host = types.Host(
