@@ -1,31 +1,18 @@
+from __future__ import annotations
+
 import enum
 import functools
 from datetime import timedelta
 from ipaddress import IPv4Address, IPv6Address
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Iterable,
-    Iterator,
-    List,
-    Mapping,
-    MutableSet,
-    Optional,
-    Sequence,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-    overload,
-)
+from typing import Any, Tuple, TypeVar, Union, overload
 
 import attr
 import psutil
 from attr import validators
 
 from . import colors, compat, pg, utils
-from .config import Configuration, Flag
+from .compat import Callable, Iterable, Iterator, Mapping, MutableSet, Sequence
+from .config import Configuration, Flag, HeaderSection, UISection
 
 
 class Pct(float):
@@ -57,10 +44,10 @@ def enum_next(e: E) -> E:
 
 @attr.s(auto_attribs=True, frozen=True, slots=True)
 class Filters:
-    dbname: Optional[str] = None
+    dbname: str | None = None
 
     @classmethod
-    def from_options(cls, filters: Sequence[str]) -> "Filters":
+    def from_options(cls, filters: Sequence[str]) -> Filters:
         fields = compat.fields_dict(cls)
         attrs = {}
         for f in filters:
@@ -89,7 +76,7 @@ class SortKey(enum.Enum):
     duration = enum.auto()
 
     @classmethod
-    def default(cls) -> "SortKey":
+    def default(cls) -> SortKey:
         return cls.duration
 
 
@@ -100,7 +87,7 @@ class QueryMode(enum.Enum):
     blocking = "blocking queries"
 
     @classmethod
-    def default(cls) -> "QueryMode":
+    def default(cls) -> QueryMode:
         return cls.activities
 
 
@@ -155,14 +142,14 @@ class Column:
     key: str = attr.ib(repr=False)
     name: str
     mandatory: bool = False
-    sort_key: Optional[SortKey] = None
+    sort_key: SortKey | None = None
     min_width: int = attr.ib(default=0, repr=False)
-    max_width: Optional[int] = attr.ib(default=None, repr=False)
+    max_width: int | None = attr.ib(default=None, repr=False)
     justify: str = attr.ib(
         "left", validator=validators.in_(["left", "center", "right"])
     )
     transform: Callable[[Any], str] = attr.ib(default=if_none(""), repr=False)
-    color_key: Union[str, Callable[[Any], str]] = attr.ib(
+    color_key: str | Callable[[Any], str] = attr.ib(
         default=_color_key_marker, repr=False
     )
 
@@ -206,11 +193,82 @@ class Column:
         return self.color_key
 
 
+@attr.s(auto_attribs=True, slots=True)
+class UIHeader:
+    """Configuration for the header of the UI."""
+
+    show_instance: bool = True
+    show_system: bool = True
+    show_workers: bool = True
+
+    @classmethod
+    def make(cls, config: HeaderSection | None, **options: bool | None) -> UIHeader:
+        """Return a UIHeader built from configuration and command-line options.
+
+        Command-line options take precedence over configuration.
+
+        >>> config = HeaderSection(show_instance=False, show_system=False)
+        >>> UIHeader.make(config, show_instance=True, show_workers=False)
+        UIHeader(show_instance=True, show_system=False, show_workers=False)
+        """
+        values = {}
+        if config is not None:
+            values.update(attr.asdict(config))
+        values.update({k: v for k, v in options.items() if v is not None})
+        return cls(**values)
+
+    def toggle_system(self) -> None:
+        """Toggle the 'show_system' attribute.
+
+        >>> h = UIHeader()
+        >>> h.show_system
+        True
+        >>> h.toggle_system()
+        >>> h.show_system
+        False
+        >>> h.toggle_system()
+        >>> h.show_system
+        True
+        """
+        self.show_system = not self.show_system
+
+    def toggle_instance(self) -> None:
+        """Toggle the 'show_instance' attribute.
+
+        >>> h = UIHeader()
+        >>> h.show_instance
+        True
+        >>> h.toggle_instance()
+        >>> h.show_instance
+        False
+        >>> h.toggle_instance()
+        >>> h.show_instance
+        True
+        """
+        self.show_instance = not self.show_instance
+
+    def toggle_workers(self) -> None:
+        """Toggle the 'show_workers' attribute.
+
+        >>> h = UIHeader()
+        >>> h.show_workers
+        True
+        >>> h.toggle_workers()
+        >>> h.show_workers
+        False
+        >>> h.toggle_workers()
+        >>> h.show_workers
+        True
+        """
+        self.show_workers = not self.show_workers
+
+
 @attr.s(auto_attribs=True, slots=True)
 class UI:
     """State of the UI."""
 
-    columns_by_querymode: Mapping[QueryMode, Tuple[Column, ...]]
+    header: UIHeader
+    columns_by_querymode: Mapping[QueryMode, tuple[Column, ...]]
     min_duration: float = 0.0
     duration_mode: DurationMode = attr.ib(
         default=DurationMode.query, converter=DurationMode
@@ -218,24 +276,25 @@ class UI:
     wrap_query: bool = False
     sort_key: SortKey = attr.ib(default=SortKey.default(), converter=SortKey)
     query_mode: QueryMode = attr.ib(default=QueryMode.activities, converter=QueryMode)
-    refresh_time: Union[float, int] = 2
+    refresh_time: float | int = 2
     in_pause: bool = False
-    interactive_timeout: Optional[int] = None
-    show_instance_info_in_header: bool = True
-    show_system_info_in_header: bool = True
-    show_worker_info_in_header: bool = True
+    interactive_timeout: int | None = None
 
     @classmethod
     def make(
         cls,
-        config: Optional[Configuration] = None,
+        header: UIHeader | None = None,
+        config: Configuration | None = None,
         flag: Flag = Flag.all(),
         *,
         max_db_length: int = 16,
         filters: Filters = NO_FILTER,
         **kwargs: Any,
-    ) -> "UI":
-        possible_columns: Dict[str, Column] = {}
+    ) -> UI:
+        if header is None:
+            header = UIHeader.make(config.header() if config else None)
+
+        possible_columns: dict[str, Column] = {}
 
         def add_column(key: str, name: str, **kwargs: Any) -> None:
             if config is not None:
@@ -244,6 +303,7 @@ class UI:
                 except KeyError:
                     pass
                 else:
+                    assert isinstance(cfg, UISection), cfg
                     if cfg.width is not None:
                         kwargs["min_width"] = kwargs["max_width"] = cfg.width
             assert key not in possible_columns, f"duplicated key {key}"
@@ -278,7 +338,7 @@ class UI:
                 key="database",
                 name="DATABASE(*)" if filters.dbname else "DATABASE",
                 min_width=max_db_length,
-                transform=functools.lru_cache()(
+                transform=functools.lru_cache(
                     lambda v: utils.ellipsis(v, width=16) if v else "",
                 ),
                 sort_key=None,
@@ -388,7 +448,7 @@ class UI:
                 transform=utils.naturalsize,
             )
 
-        columns_key_by_querymode: Mapping[QueryMode, List[str]] = {
+        columns_key_by_querymode: Mapping[QueryMode, list[str]] = {
             QueryMode.activities: [
                 "pid",
                 "database",
@@ -442,7 +502,7 @@ class UI:
                     pass
 
         columns_by_querymode = {qm: tuple(make_columns_for(qm)) for qm in QueryMode}
-        return cls(columns_by_querymode=columns_by_querymode, **kwargs)
+        return cls(header=header, columns_by_querymode=columns_by_querymode, **kwargs)
 
     def interactive(self) -> bool:
         return self.interactive_timeout is not None
@@ -513,51 +573,6 @@ class UI:
         """
         self.in_pause = not self.in_pause
 
-    def toggle_system_info_in_header(self) -> None:
-        """Toggle the 'show_system_info_in_header' attribute.
-
-        >>> ui = UI.make()
-        >>> ui.show_system_info_in_header
-        True
-        >>> ui.toggle_system_info_in_header()
-        >>> ui.show_system_info_in_header
-        False
-        >>> ui.toggle_system_info_in_header()
-        >>> ui.show_system_info_in_header
-        True
-        """
-        self.show_system_info_in_header = not self.show_system_info_in_header
-
-    def toggle_instance_info_in_header(self) -> None:
-        """Toggle the 'show_instance_info_in_header' attribute.
-
-        >>> ui = UI.make()
-        >>> ui.show_instance_info_in_header
-        True
-        >>> ui.toggle_instance_info_in_header()
-        >>> ui.show_instance_info_in_header
-        False
-        >>> ui.toggle_instance_info_in_header()
-        >>> ui.show_instance_info_in_header
-        True
-        """
-        self.show_instance_info_in_header = not self.show_instance_info_in_header
-
-    def toggle_worker_info_in_header(self) -> None:
-        """Toggle the 'show_worker_info_in_header' attribute.
-
-        >>> ui = UI.make()
-        >>> ui.show_worker_info_in_header
-        True
-        >>> ui.toggle_worker_info_in_header()
-        >>> ui.show_worker_info_in_header
-        False
-        >>> ui.toggle_worker_info_in_header()
-        >>> ui.show_worker_info_in_header
-        True
-        """
-        self.show_worker_info_in_header = not self.show_worker_info_in_header
-
     def evolve(self, **changes: Any) -> None:
         """Return a new UI with 'changes' applied.
 
@@ -604,7 +619,7 @@ class UI:
         else:
             raise ValueError(key)
 
-    def columns(self) -> Tuple[Column, ...]:
+    def columns(self) -> tuple[Column, ...]:
         """Return the tuple of Column for current mode.
 
         >>> flag = Flag.PID | Flag.DATABASE | Flag.APPNAME | Flag.RELATION
@@ -631,18 +646,18 @@ class SwapInfo:
     total: int
 
     @classmethod
-    def default(cls) -> "SwapInfo":
+    def default(cls) -> SwapInfo:
         return cls(0, 0, 0)
 
     @property
-    def pct_used(self) -> Optional[Pct]:
+    def pct_used(self) -> Pct | None:
         if self.total == 0:
             # account for the zero swap case (#318)
             return None
         return Pct(self.used * 100 / self.total)
 
     @property
-    def pct_free(self) -> Optional[Pct]:
+    def pct_free(self) -> Pct | None:
         if self.total == 0:
             # account for the zero swap case (#318)
             return None
@@ -657,23 +672,23 @@ class MemoryInfo:
     total: int
 
     @classmethod
-    def default(cls) -> "MemoryInfo":
+    def default(cls) -> MemoryInfo:
         return cls(0, 0, 0, 0)
 
     @property
-    def pct_used(self) -> Optional[Pct]:
+    def pct_used(self) -> Pct | None:
         if self.total == 0:
             return None
         return Pct(self.used * 100 / self.total)
 
     @property
-    def pct_free(self) -> Optional[Pct]:
+    def pct_free(self) -> Pct | None:
         if self.total == 0:
             return None
         return Pct(self.free * 100 / self.total)
 
     @property
-    def pct_bc(self) -> Optional[Pct]:
+    def pct_bc(self) -> Pct | None:
         if self.total == 0:
             return None
         return Pct(self.buff_cached * 100 / self.total)
@@ -686,7 +701,7 @@ class LoadAverage:
     avg15: float
 
     @classmethod
-    def default(cls) -> "LoadAverage":
+    def default(cls) -> LoadAverage:
         return cls(0.0, 0.0, 0.0)
 
 
@@ -694,10 +709,9 @@ class LoadAverage:
 class IOCounter:
     count: int
     bytes: int
-    chars: int = 0
 
     @classmethod
-    def default(cls) -> "IOCounter":
+    def default(cls) -> IOCounter:
         return cls(0, 0)
 
 
@@ -714,18 +728,18 @@ class SystemInfo:
     def default(
         cls,
         *,
-        memory: Optional[MemoryInfo] = None,
-        swap: Optional[SwapInfo] = None,
-        load: Optional[LoadAverage] = None,
-    ) -> "SystemInfo":
+        memory: MemoryInfo | None = None,
+        swap: SwapInfo | None = None,
+        load: LoadAverage | None = None,
+    ) -> SystemInfo:
         """Zero-value builder.
 
         >>> SystemInfo.default()  # doctest: +NORMALIZE_WHITESPACE
         SystemInfo(memory=MemoryInfo(used=0, buff_cached=0, free=0, total=0),
                    swap=SwapInfo(used=0, free=0, total=0),
                    load=LoadAverage(avg1=0.0, avg5=0.0, avg15=0.0),
-                   io_read=IOCounter(count=0, bytes=0, chars=0),
-                   io_write=IOCounter(count=0, bytes=0, chars=0), max_iops=0)
+                   io_read=IOCounter(count=0, bytes=0),
+                   io_write=IOCounter(count=0, bytes=0), max_iops=0)
         """
         return cls(
             memory or MemoryInfo.default(),
@@ -760,6 +774,8 @@ class ServerInformation:
     total_size: int
     blks_read: int
     blks_hit: int
+    xact_commit: int
+    xact_rollback: int
     max_dbname_length: int
     uptime: timedelta
     epoch: int  # an epoch,  used for the calculation of the tps & size_evolution
@@ -770,19 +786,19 @@ class ServerInformation:
     total: int
     waiting: int
     max_connections: int
-    autovacuum_workers: Optional[int]
+    autovacuum_workers: int | None
     autovacuum_max_workers: int
-    logical_replication_workers: Optional[int]
-    parallel_workers: Optional[int]
-    max_logical_replication_workers: Optional[int]
-    max_parallel_workers: Optional[int]
-    max_worker_processes: Optional[int]
-    max_wal_senders: Optional[int]
-    max_replication_slots: Optional[int]
-    wal_senders: Optional[int]
-    wal_receivers: Optional[int]
-    replication_slots: Optional[int]
-    temporary_file: Optional[TempFileInfo]
+    logical_replication_workers: int | None
+    parallel_workers: int | None
+    max_logical_replication_workers: int | None
+    max_parallel_workers: int | None
+    max_worker_processes: int | None
+    max_wal_senders: int | None
+    max_replication_slots: int | None
+    wal_senders: int | None
+    wal_receivers: int | None
+    replication_slots: int | None
+    temporary_file: TempFileInfo | None
     # Computed in data.pg_get_server_information()
     size_evolution: float
     tps: int
@@ -790,12 +806,15 @@ class ServerInformation:
     update_per_second: int
     delete_per_second: int
     tuples_returned_per_second: int
-    cache_hit_ratio_last_snap: Optional[Pct] = attr.ib(
+    cache_hit_ratio_last_snap: Pct | None = attr.ib(
+        converter=attr.converters.optional(Pct)
+    )
+    rollback_ratio_last_snap: Pct | None = attr.ib(
         converter=attr.converters.optional(Pct)
     )
 
     @property
-    def worker_processes(self) -> Optional[int]:
+    def worker_processes(self) -> int | None:
         if self.parallel_workers is None and self.logical_replication_workers is None:
             return None
         else:
@@ -840,24 +859,24 @@ def locktype(value: str) -> LockType:
 class BaseProcess:
     pid: int
     application_name: str
-    database: Optional[str]
+    database: str | None
     user: str
-    client: Union[None, IPv4Address, IPv6Address]
-    duration: Optional[float]
+    client: None | IPv4Address | IPv6Address
+    duration: float | None
     state: str
-    query: Optional[str]
-    encoding: Optional[str]
-    query_leader_pid: Optional[int]
+    query: str | None
+    encoding: str | None
+    query_leader_pid: int | None
     is_parallel_worker: bool
 
     _P = TypeVar("_P", bound="BaseProcess")
 
     @classmethod
     def from_bytes(
-        cls: Type[_P],
+        cls: type[_P],
         server_encoding: bytes,
         *,
-        encoding: Optional[Union[str, bytes]],
+        encoding: str | bytes | None,
         **kwargs: Any,
     ) -> _P:
         if encoding is None:
@@ -876,8 +895,8 @@ class BaseProcess:
 class RunningProcess(BaseProcess):
     """Process for a running query."""
 
-    wait: Union[bool, None, str]
-    query_leader_pid: Optional[int]
+    wait: bool | None | str
+    query_leader_pid: int | None
     is_parallel_worker: bool
 
 
@@ -892,7 +911,7 @@ class WaitingProcess(BaseProcess):
     relation: str
 
     # TODO: update queries to select/compute these column.
-    query_leader_pid: Optional[int] = attr.ib(default=None, init=False)
+    query_leader_pid: int | None = attr.ib(default=None, init=False)
     is_parallel_worker: bool = attr.ib(default=False, init=False)
 
 
@@ -905,26 +924,26 @@ class BlockingProcess(BaseProcess):
     mode: str
     type: LockType = attr.ib(converter=locktype)
     relation: str
-    wait: Union[bool, None, str]
+    wait: bool | None | str
 
     # TODO: update queries to select/compute these column.
-    query_leader_pid: Optional[int] = attr.ib(default=None, init=False)
+    query_leader_pid: int | None = attr.ib(default=None, init=False)
     is_parallel_worker: bool = attr.ib(default=False, init=False)
 
 
 @attr.s(auto_attribs=True, frozen=True, slots=True)
 class SystemProcess:
-    meminfo: Tuple[int, ...]
+    meminfo: tuple[int, ...]
     io_read: IOCounter
     io_write: IOCounter
     io_time: float
     mem_percent: float
     cpu_percent: float
-    cpu_times: Tuple[float, ...]
+    cpu_times: tuple[float, ...]
     read_delta: float
     write_delta: float
     io_wait: bool
-    psutil_proc: Optional[psutil.Process]
+    psutil_proc: psutil.Process | None
 
 
 @attr.s(auto_attribs=True, frozen=True, slots=True)
@@ -937,8 +956,8 @@ class LocalRunningProcess(RunningProcess):
 
     @classmethod
     def from_process(
-        cls, process: RunningProcess, **kwargs: Union[float, str]
-    ) -> "LocalRunningProcess":
+        cls, process: RunningProcess, **kwargs: float | str
+    ) -> LocalRunningProcess:
         return cls(**dict(attr.asdict(process), **kwargs))
 
 
@@ -1043,8 +1062,8 @@ class SelectableProcesses:
 
     """
 
-    items: List[BaseProcess]
-    focused: Optional[int] = None
+    items: list[BaseProcess]
+    focused: int | None = None
     pinned: MutableSet[int] = attr.ib(default=attr.Factory(set))
 
     def __len__(self) -> int:
@@ -1054,20 +1073,16 @@ class SelectableProcesses:
         return iter(self.items)
 
     @overload
-    def __getitem__(self, i: int) -> BaseProcess:
-        ...
+    def __getitem__(self, i: int) -> BaseProcess: ...
 
     @overload
-    def __getitem__(self, s: slice) -> List[BaseProcess]:
-        ...
+    def __getitem__(self, s: slice) -> list[BaseProcess]: ...
 
-    def __getitem__(
-        self, val: Union[int, slice]
-    ) -> Union[BaseProcess, List[BaseProcess]]:
+    def __getitem__(self, val: int | slice) -> BaseProcess | list[BaseProcess]:
         return self.items[val]
 
     @property
-    def selected(self) -> List[int]:
+    def selected(self) -> list[int]:
         if self.pinned:
             return list(self.pinned)
         elif self.focused:
@@ -1082,7 +1097,7 @@ class SelectableProcesses:
     def set_items(self, new_items: Sequence[BaseProcess]) -> None:
         self.items[:] = list(new_items)
 
-    def position(self) -> Optional[int]:
+    def position(self) -> int | None:
         if self.focused is None:
             return None
         for idx, proc in enumerate(self.items):
