@@ -1,14 +1,16 @@
+from __future__ import annotations
+
 import functools
 import inspect
 import itertools
 from textwrap import TextWrapper, dedent
-from typing import Any, Callable, Iterable, Iterator, List, Optional, Sequence, Tuple
+from typing import Any
 
 from blessed import Terminal
 
 from . import colors, utils
 from .activities import sorted as sorted_processes
-from .compat import link
+from .compat import Callable, Iterable, Iterator, Sequence, link
 from .keys import BINDINGS, EXIT_KEY
 from .keys import HELP as HELP_KEY
 from .keys import (
@@ -47,7 +49,7 @@ class line_counter:
 
 
 @functools.lru_cache(maxsize=512)
-def shorten(term: Terminal, text: str, width: Optional[int] = None) -> str:
+def shorten(term: Terminal, text: str, width: int | None = None) -> str:
     r"""Truncate 'text' to fit in the given 'width' (or term.width).
 
     This is similar to textwrap.shorten() but sequence-aware.
@@ -157,7 +159,7 @@ def header(
     host: Host,
     pg_version: str,
     server_information: ServerInformation,
-    system_info: Optional[SystemInfo] = None,
+    system_info: SystemInfo | None = None,
 ) -> Iterator[str]:
     @functools.singledispatch
     def render(x: Any) -> str:
@@ -188,7 +190,7 @@ def header(
         return f"{term.bold_green(hbytes)} - {term.bold_green(counts)}"
 
     def render_columns(
-        columns: Sequence[List[str]], *, delimiter: str = f"{term.blue(',')} "
+        columns: Sequence[list[str]], *, delimiter: str = f"{term.blue(',')} "
     ) -> Iterator[str]:
         column_widths = [
             max(len(column_row) for column_row in column) for column in columns
@@ -230,12 +232,13 @@ def header(
     size_ev = f"{utils.naturalsize(si.size_evolution)}/s"
     uptime = utils.naturaltimedelta(si.uptime)
 
-    if ui.show_instance_info_in_header:
+    if ui.header.show_instance:
         # First rows are always displayed, as the underlying data is always available.
         columns = [
             [f"* Global: {render(uptime)} uptime"],
             [f"{render(total_size)} dbs size - {render(size_ev)} growth"],
             [f"{render(si.cache_hit_ratio_last_snap)} cache hit ratio"],
+            [f"{render(si.rollback_ratio_last_snap)} rollback ratio"],
         ]
         yield from render_columns(columns)
 
@@ -265,7 +268,7 @@ def header(
             [f"{render(temp_size)} temp size"],
         ]
         yield from render_columns(columns)
-    if ui.show_worker_info_in_header:
+    if ui.header.show_workers:
         columns = [
             [
                 f"* Worker processes: {render(si.worker_processes)}/{render(si.max_worker_processes)} total"
@@ -292,7 +295,7 @@ def header(
         yield from render_columns(columns)
 
     # System information, only available in "local" mode.
-    if system_info is not None and ui.show_system_info_in_header:
+    if system_info is not None and ui.header.show_system:
         used, bc, free, total = (
             utils.naturalsize(system_info.memory.used),
             utils.naturalsize(system_info.memory.buff_cached),
@@ -401,7 +404,7 @@ def processes_rows(
     ui: UI,
     processes: SelectableProcesses,
     maxlines: int,
-    width: Optional[int],
+    width: int | None,
 ) -> Iterator[str]:
     """Display table rows with processes information."""
 
@@ -448,7 +451,7 @@ def processes_rows(
             color_type = "yellow"
         else:
             color_type = "default"
-        text: List[str] = []
+        text: list[str] = []
         for column in ui.columns():
             field = column.key
             if field != "query":
@@ -471,13 +474,13 @@ def processes_rows(
         yield from (" ".join(text) + term.normal).splitlines()
 
 
-def footer_message(term: Terminal, message: str, width: Optional[int] = None) -> None:
+def footer_message(term: Terminal, message: str, width: int | None = None) -> None:
     if width is None:
         width = term.width
     print(term.center(message[:width]) + term.normal, end="")
 
 
-def footer_help(term: Terminal, width: Optional[int] = None) -> None:
+def footer_help(term: Terminal, width: int | None = None) -> None:
     """Footer line with help keys."""
     query_modes_help = [
         ("/".join(keys[:-1]), qm.value) for qm, keys in KEYS_BY_QUERYMODE.items()
@@ -492,7 +495,7 @@ def footer_help(term: Terminal, width: Optional[int] = None) -> None:
 
 
 def render_footer(
-    term: Terminal, footer_values: List[Tuple[str, str]], width: Optional[int]
+    term: Terminal, footer_values: list[tuple[str, str]], width: int | None
 ) -> None:
     if width is None:
         width = term.width
@@ -513,12 +516,12 @@ def render_footer(
     print(term.ljust(row, width=width, fillchar=term.cyan_reverse(" ")), end="")
 
 
-def footer_interative_help(term: Terminal, width: Optional[int] = None) -> None:
+def footer_interative_help(term: Terminal, width: int | None = None) -> None:
     """Footer line with help keys for interactive mode."""
     assert PROCESS_PIN.name is not None
     footer_values = [
         (PROCESS_CANCEL, "cancel current query"),
-        (PROCESS_KILL, "terminate current query"),
+        (PROCESS_KILL, "terminate underlying session"),
         (PROCESS_PIN.name, PROCESS_PIN.description),
         ("Other", "back to activities"),
         (EXIT_KEY.value, EXIT_KEY.description),
@@ -534,14 +537,14 @@ def screen(
     pg_version: str,
     server_information: ServerInformation,
     activity_stats: ActivityStats,
-    message: Optional[str],
+    message: str | None,
     render_header: bool = True,
     render_footer: bool = True,
-    width: Optional[int] = None,
+    width: int | None = None,
 ) -> None:
     """Display the screen."""
 
-    system_info: Optional[SystemInfo]
+    system_info: SystemInfo | None
     if isinstance(activity_stats, tuple):
         processes, system_info = activity_stats
     else:
